@@ -1,5 +1,5 @@
 import pickle
-from typing import Optional, List, Dict
+from typing import Optional, List, Dict, Callable
 
 import ipywidgets as widgets
 import k3d
@@ -9,7 +9,7 @@ from ipywidgets import Play
 from k3d.plot import Plot
 
 from .jointset import JointSet, MuPoTSJoints, OpenPoseJoints, CocoExJoints, PanopticJoints, Common14Joints
-from .objects import Skeleton, DrawableSkeleton, Color
+from .objects import Skeleton, DrawableSkeleton, Color, Positions
 
 
 class FrameUpdater:
@@ -49,7 +49,11 @@ class SkeletonVisualizer:
     def visualize(self, skeletons: np.ndarray, colors: List[Color] = None, include_names: bool = False) -> None:
         self.__assert_skeleton_shapes(skeletons)
         colors = self.__init_colors(skeletons.shape[0], colors)
-        self.plot = self.__create_skeleton_plot(skeletons, colors, include_names)
+        if include_names:
+            skeleton_converter = Skeleton.to_drawable_skeleton_with_names
+        else:
+            skeleton_converter = Skeleton.to_drawable_skeleton
+        self.__create_skeleton_plot(skeletons, skeleton_converter, colors)
         self.plot.display()
         if include_names:
             self.__link_joint_name_visibility_with_checkbox()
@@ -68,36 +72,71 @@ class SkeletonVisualizer:
             self, frames: np.ndarray, colors: List[Color] = None,
             fps: int = 15, include_names: bool = False) -> None:
         assert len(frames.shape) == 4
-        self.__assert_skeleton_shapes(frames[0])
-        colors = self.__init_colors(frames[0].shape[0], colors)
-
-        self.plot = k3d.plot(camera_auto_fit=False)
-        self.__adjust_camera_starting_position(frames[0], self.plot)
-        skeleton_part_size = self.__calculate_skeleton_part_size(frames[0])
-
-        frames = np.swapaxes(frames, 0, 1)
-        skeleton_timestamps: List[Dict[str, np.ndarray]] = [
-            {
-                str(timestamp_index): frames[current_frame_index][timestamp_index]
-                for timestamp_index in range(len(frames[current_frame_index]))
-            }
-            for current_frame_index in range(len(frames))
-        ]
-
-        for skeleton in map(
-                lambda skeleton_color_tuple: Skeleton(
-                    joint_positions=skeleton_color_tuple[0],
-                    joint_set=self.joint_set,
-                    part_size=skeleton_part_size,
-                    color=skeleton_color_tuple[1]),
-                zip(skeleton_timestamps, colors)):
-            drawable_skeleton = skeleton.to_drawable_skeleton_for_video()
-            self.plot += drawable_skeleton
-            self.skeletons.append(drawable_skeleton)
-
+        first_frame = frames[0]
+        self.__assert_skeleton_shapes(first_frame)
+        colors = self.__init_colors(first_frame.shape[0], colors)
+        skeleton_timestamps: List[Dict[str, np.ndarray]] = self.__get_skeleton_positions_timestamps(frames)
+        self.__create_skeleton_plot(
+            skeletons=first_frame,
+            skeleton_converter=Skeleton.to_drawable_skeleton_for_video,
+            colors=colors, positions=skeleton_timestamps)
         self.plot.display()
         self.__display_video_player(fps, frames)
 
+    @staticmethod
+    def __get_skeleton_positions_timestamps(frames: np.ndarray) -> List[Dict[str, np.ndarray]]:
+        frames_swapped = np.swapaxes(frames, 0, 1)
+        return [
+            {
+                str(timestamp_index): frames_swapped[current_frame_index][timestamp_index]
+                for timestamp_index in range(len(frames_swapped[current_frame_index]))
+            }
+            for current_frame_index in range(len(frames_swapped))
+        ]
+
+    def __create_skeleton_plot(self, skeletons: np.ndarray, skeleton_converter: Callable[[Skeleton], DrawableSkeleton],
+                               colors: List[Color], positions: Optional[Positions] = None) -> None:
+        self.__init_skeleton_plot(skeletons)
+        skeleton_part_size = self.__calculate_skeleton_part_size(skeletons)
+        if positions is None:
+            positions = skeletons
+        skeletons: List[Skeleton] = self.__get_skeletons(positions, colors, skeleton_part_size)
+        drawable_skeletons: List[DrawableSkeleton] = [skeleton_converter(skeleton) for skeleton in skeletons]
+        self.__add_skeletons_to_plot(drawable_skeletons)
+
+    def __init_skeleton_plot(self, skeletons: np.ndarray):
+        self.plot = k3d.plot(camera_auto_fit=False)
+        centroid: np.ndarray = np.average(np.average(skeletons, axis=0), axis=0)
+        camera_up_vector: np.ndarray = np.zeros(shape=(3,))
+        for line_indices in self.joint_set.vertically_aligned_line_indices:
+            camera_up_vector += np.sum(
+                skeletons[:, line_indices[1]] - skeletons[:, line_indices[0]],
+                axis=0)
+        camera_up_vector /= np.linalg.norm(camera_up_vector, ord=2)
+        self.plot.camera = [
+            0, 0, 0,
+            centroid[0], centroid[1], centroid[2],
+            camera_up_vector[0], camera_up_vector[1], camera_up_vector[2]]
+
+    def __calculate_skeleton_part_size(self, skeletons: np.ndarray) -> float:
+        max_values = [abs(skeleton).max() for skeleton in skeletons]
+        return (min(max_values) / 100.0) * self.size_scalar
+
+    def __get_skeletons(self, positions: List[Positions],
+                        colors: List[Color], skeleton_part_size: float) -> List[Skeleton]:
+        return [skeleton for skeleton in map(
+                lambda position_color_tuple: Skeleton(
+                    joint_positions=position_color_tuple[0],
+                    joint_set=self.joint_set,
+                    part_size=skeleton_part_size,
+                    color=position_color_tuple[1]),
+                zip(positions, colors))]
+
+    def __add_skeletons_to_plot(self, skeletons: List[DrawableSkeleton]):
+        for drawable_skeleton in skeletons:
+            self.plot += drawable_skeleton
+            self.skeletons.append(drawable_skeleton)
+
     def __display_video_player(self, fps, frames):
         video_player: Play = Play(
             value=0,
@@ -105,20 +144,14 @@ class SkeletonVisualizer:
             step=1, interval=1000 / fps,
             description='Press play', disabled=False
         )
-
         frame_updater = FrameUpdater(self.skeletons, frames, self.joint_set, video_player)
-
-        frame_slider = widgets.IntSlider(value=0, min=0, max=frames.shape[0] - 1, step=1, description='Frame')
-
+        frame_slider = widgets.IntSlider(value=0, min=0, max=frames.shape[1] - 1, step=1, description='Frame')
         next_frame_button = widgets.Button(description='Next frame')
         next_frame_button.on_click(frame_updater.update_to_next_frame)
-
         previous_frame_button = widgets.Button(description='Prev frame')
         previous_frame_button.on_click(frame_updater.update_to_previous_frame)
-
         widgets.jslink((video_player, 'value'), (frame_slider, 'value'))
         widgets.jslink((video_player, 'value'), (self.plot, 'time'))
-
         display(widgets.HBox([video_player, previous_frame_button, frame_slider, next_frame_button]))
 
     @staticmethod
@@ -136,43 +169,6 @@ class SkeletonVisualizer:
             'The number of joints of skeletons and the number of joints in the specified joint set must be the same.'
         assert skeletons.shape[2] == 3, 'The skeleton joint coordinates must be 3 dimensional'
 
-    def __create_skeleton_plot(self, skeletons: np.ndarray,
-                               colors: List[Color], include_names: bool = False) -> Optional[Plot]:
-        skeleton_part_size = self.__calculate_skeleton_part_size(skeletons)
-        skeleton_plot = k3d.plot(camera_auto_fit=False)
-        self.__adjust_camera_starting_position(skeletons, skeleton_plot)
-        for skeleton in map(
-                lambda skeleton_color_tuple: Skeleton(
-                    joint_positions=skeleton_color_tuple[0],
-                    joint_set=self.joint_set,
-                    part_size=skeleton_part_size,
-                    color=skeleton_color_tuple[1]),
-                zip(skeletons, colors)):
-            if include_names:
-                drawable_skeleton = skeleton.to_drawable_skeleton_with_names()
-            else:
-                drawable_skeleton = skeleton.to_drawable_skeleton()
-            skeleton_plot += drawable_skeleton
-            self.skeletons.append(drawable_skeleton)
-        return skeleton_plot
-
-    def __calculate_skeleton_part_size(self, skeletons: np.ndarray) -> float:
-        max_values = [abs(skeleton).max() for skeleton in skeletons]
-        return (min(max_values) / 100.0) * self.size_scalar
-
-    def __adjust_camera_starting_position(self, skeletons: np.ndarray, skeleton_plot: Plot) -> None:
-        centroid: np.ndarray = np.average(np.average(skeletons, axis=0), axis=0)
-        camera_up_vector: np.ndarray = np.zeros(shape=(3,))
-        for line_indices in self.joint_set.vertically_aligned_line_indices:
-            camera_up_vector += np.sum(
-                skeletons[:, line_indices[1]] - skeletons[:, line_indices[0]],
-                axis=0)
-        camera_up_vector /= np.linalg.norm(camera_up_vector, ord=2)
-        skeleton_plot.camera = [
-            0, 0, 0,
-            centroid[0], centroid[1], centroid[2],
-            camera_up_vector[0], camera_up_vector[1], camera_up_vector[2]]
-
     def __link_joint_name_visibility_with_checkbox(self) -> None:
         for skeleton in self.skeletons:
             for joint_name in skeleton.joint_names:
